<!DOCTYPE html>

{% extends "layout.html.jinja" %}

{%block content %}

<table class="table table-bordered table-striped ">
  <thead>
    <tr>
      <th>{{_('Name')}}</th>
      <th>{{_('Value')}}</th>
    </tr>
  </thead>

  <tbody>
    {% for info in application_info %}
    <tr>
      <td>{{info[0]}}</td>
      <td>{{info[1]}}</td>
    </tr>
    {% endfor %}
  </tbody>
</table>
{% endblock %}
