<!DOCTYPE html>

<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">

    <title>{{ _('Odoo PoS Drivers')}}</title>

    <!-- from : https://code.jquery.com/jquery-3.7.1.min.js -->
    <script src="{{ url_for('static', filename='js/jquery.min.js') }}"></script>

    <!-- from : https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/ -->
    <script src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
    <link href="{{url_for('static', filename='css/bootstrap.min.css')}}" rel="stylesheet" />

    <script src="{{ url_for('static', filename='js/webcam-easy.js') }}"></script>

    <link href="{{url_for('static', filename='css/odoo_driver.css')}}" rel="stylesheet" />

  </head>
  <body>
    <nav class="navbar navbar-expand-lg bg-success navbar-dark">
      <div class="container-fluid">
        <a class="navbar-brand" >Odoo POS Driver</a>
          <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarSupportedContent" aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
            <span class="navbar-toggler-icon"></span>
          </button>
        <div class="collapse navbar-collapse" id="navbarSupportedContent">
          <ul class="navbar-nav me-auto mb-2 mb-lg-0">
            <li class="nav-item">
              <a class="nav-link {%if request.endpoint == 'home' %}active{% endif %}" href="{{url_for('home')}}">{{_('Home Page')}}</a>
            </li>
            <li class="nav-item">
              <a class="nav-link {%if request.endpoint == 'usb_devices' %}active{% endif %}" href="{{url_for('usb_devices')}}">{{_('USB Devices')}}</a>
            </li>
            <li class="nav-item">
              <a class="nav-link {%if request.endpoint == 'system' %}active{% endif %}" href="{{url_for('system')}}">{{_('System')}}</a>
            </li>
            <li class="nav-item">
              <a class="nav-link {%if request.endpoint == 'application' %}active{% endif %}" href="{{url_for('application')}}">{{_('Application')}}</a>
            </li>
            <li class="nav-item">
              <a class="nav-link {%if request.endpoint == 'photo_booth' %}active{% endif %}" href="{{url_for('photo_booth')}}">{{_('Photo Booth')}}</a>
            </li>
          </ul>
        </div>
      </div>
    </nav>
    {% block content %}{% endblock %}
  </body>
</html>
