import curses.ascii

STX = chr(curses.ascii.STX)
ETX = chr(curses.ascii.ETX)
EOT = chr(curses.ascii.EOT)
ENQ = chr(curses.ascii.ENQ)
ACK = chr(curses.ascii.ACK)
NAK = chr(curses.ascii.NAK)
ESC = chr(curses.ascii.ESC)

FF = chr(curses.ascii.FF)
US = chr(curses.ascii.US)
